package be.AM4DP;

import java.util.logging.Logger;

import be.ASPL.presentationlayer.StringValue;
import be.ASPL.presentationlayer.ASPLException;
import be.ASPL.presentationlayer.InitialPresentation;
import be.SIRAPRISE.client.ConnectionClosedException;
import be.SIRAPRISE.client.DBConnection;
import be.SIRAPRISE.client.jsba.DmlCommand;
import be.SIRAPRISE.messages.ErrorMessageException;
import be.SIRAPRISE.util.DDLCapture;

/**
 * Function to delete a job. The function is intended to be invoked only with a pre-selected job title. It does not have its own selection window.
 * 
 * @author Erwin
 */
@InitialPresentation(initialPresentationClass = JobsDeleteConfirmPresentation.class)
public class JobsDelete extends JobsWebClientFunction {

	/**
	 * 
	 */
	private boolean leave = false;

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doInitialLogic()
	 */
	public void doInitialLogic ( ) {
		final StringValue presentationValue = (StringValue) getPresentationValue(NM_JOB);
		Logger.getLogger(JobsDelete.class.getName()).info(presentationValue.getString());
		saveInFunctionContext(NM_JOB, presentationValue);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doPresentationLogic()
	 */
	public void doPresentationLogic ( ) throws ASPLException {
		// confirmation entered, delete attribute
		
		try (DBConnection dbc = getDBConnection();) {
			DmlCommand.execDmlCommandAndEndTransaction(dbc.startTransaction(getUserData().getUserID(), true, "", new byte[] {}, DDLCapture.OFF), new Job(((StringValue) getFromFunctionContextMandatory(NM_JOB)).getString().trim()).dbDeleteCommand()); //$NON-NLS-1$
			leave = true;
		} catch (ConnectionClosedException e) {
			log(e);
			throw new DBProblem(e);
		} catch (ErrorMessageException e) {
			log(e);
			throw getASPLApplicationException(e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#getPresentationFunctionDescription()
	 */
	public String getPresentationFunctionDescription () {
		return I18N.getString(getUserLocale(), "JobsDelete.Text"); //$NON-NLS-1$
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#leaveFunctionOnExit()
	 */
	public boolean leaveFunctionOnExit ( ) {
		return leave;
	}
}