package be.AM4DP;

import be.ASPL.presentationlayer.OnLinePresentationFunctionCharacteristics;
import be.ASPL.presentationlayer.StringValue;
import be.ASPL.presentationlayer.ASPLException;
import be.ASPL.presentationlayer.InitialPresentation;
import be.SIRAPRISE.client.ConnectionClosedException;
import be.SIRAPRISE.client.DBConnection;
import be.SIRAPRISE.client.NoUpdatesException;
import be.SIRAPRISE.client.jsba.DmlCommand;
import be.SIRAPRISE.client.jsba.DmlUpdateCommand;
import be.SIRAPRISE.messages.ErrorMessageException;
import be.SIRAPRISE.util.DDLCapture;

/**
 * Function to manage the details of a job. The function has no selection window of its own and is intended to be invoked only with a pre-selected job title.
 * 
 * @author Erwin
 */
@InitialPresentation(initialPresentationClass = JobsManageDetailsPresentation.class)
@OnLinePresentationFunctionCharacteristics(leaveFunctionOnExit = false, descriptionLocalizationKey = "JobsManage.Text")
public class JobsManage extends JobsWebClientFunction {

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doInitialLogic()
	 */
	public void doInitialLogic ( ) {
		Job job = new Job(((StringValue) getPresentationValue(NM_JOB)).getString().trim());
		job.setPreUpdateState();
		saveInFunctionContext(RELVARNAMES.JOBS, job);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doPresentationLogic()
	 */
	public void doPresentationLogic ( ) throws ASPLException {
		// <OK> clicked on managedetails presentation
		Job job = (Job) getFromFunctionContextMandatory(RELVARNAMES.JOBS);

		job.setJobTitle(((StringValue) getPresentationValue(NM_JOB)).getString().trim());

		try {
			DmlUpdateCommand updateCommand = job.dbUpdateCommand(RELVARNAMES.JOBS);
			
			try (DBConnection dbc = getDBConnection();) {
				DmlCommand.execDmlCommandAndEndTransaction(dbc.startTransaction(getUserData().getUserID(), true, "", new byte[] {}, DDLCapture.OFF), updateCommand); //$NON-NLS-1$

				// save new values as ones in the function context
				saveInFunctionContext(RELVARNAMES.JOBS, job);
			} catch (ConnectionClosedException e) {
				log(e);
				throw new DBProblem(e);
			} catch (ErrorMessageException e) {
				log(e);
				throw getASPLApplicationException(e);
			}
		} catch (NoUpdatesException e) {

		}
	}
}