package be.AM4DP;

import be.ASPL.presentationlayer.ASPLListFactory;
import be.ASPL.presentationlayer.InitialPresentation;
import be.ASPL.presentationlayer.OnLinePresentationFunctionCharacteristics;
import be.ASPL.presentationlayer.StringValue;
import be.ASPL.presentationlayer.ASPLException;
import be.SIRAPRISE.client.ConnectionClosedException;
import be.SIRAPRISE.client.DBConnection;
import be.SIRAPRISE.client.TransactionMode;
import be.SIRAPRISE.messages.ErrorMessageException;

/**
 * The function to produce an overview of defined jobs
 * 
 * @author Erwin
 */
@InitialPresentation(initialPresentationClass = JobsOverviewPresentation.class)
@OnLinePresentationFunctionCharacteristics(leaveFunctionOnExit = false, descriptionLocalizationKey = "JobsOverview.Text")
public class JobsOverview extends JobsWebClientFunction {

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doInitialLogic()
	 */
	public void doInitialLogic ( ) throws ASPLException {
		doPresentationLogic();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doPresentationLogic()
	 */
	public void doPresentationLogic ( ) throws ASPLException {
		String restrictCondition = ""; //$NON-NLS-1$

		StringValue namePattern = (StringValue) getPresentationValue(JobsOverviewPresentation.NAMEPATTERN);
		if (namePattern != null) {
			restrictCondition = "MATCHES(THE_STRING(NM_JOB),STRING(" + namePattern.getString() + "))"; //$NON-NLS-1$//$NON-NLS-2$
		}

		String query = RELVARNAMES.JOBS;
		if (restrictCondition.length() > 0) {
			query = "RESTRICT(" + query + "," + restrictCondition + ")"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}

		
		try (DBConnection dbc = getDBConnection();) {
			setPresentationValue(JobsOverviewPresentation.JOBLIST, ASPLListFactory.getASPLListValue(getPresentation().getPresentationField(JobsOverviewPresentation.JOBLIST), dbc.startTransaction(getUserData().getUserID(), true, "", new byte[] {}, TransactionMode.READONLY).execQueryAndEndTransaction(query), getUserData(), getUserLocale())); //$NON-NLS-1$
		} catch (ConnectionClosedException e) {
			log(e);
			throw new DBProblem(e);
		} catch (ErrorMessageException e) {
			log(e);
			throw getASPLApplicationException(e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doResumeLogic()
	 */
	public void doResumeLogic ( ) throws ASPLException {
		doPresentationLogic();
	}
}