package be.ASPL.presentationlayer;

import java.util.Locale;

import be.SIRAPRISE.util.Localizer;
import be.SIRAPRISE.util.MyMessageFormat;

/**
 * @author Erwin
 */
public abstract class ASPLException extends Exception {

	/**
	 * 
	 */
	private static final long serialVersionUID = -5872571776833187540L;

	/**
	 * 
	 */
	private String localeTextTemplateID;

	/**
	 * 
	 */
	private Localizer localizer;

	/**
	 * 
	 */
	private String[] params;

	/**
	 * Creates the ASPLException
	 * 
	 * @param localizer
	 *            The locale in which to set the message text
	 * @param localeTextTemplateID
	 *            The key that will allow the localizer to obtain the correct mesage text template
	 * @param params
	 *            The parameters to be used by the localizer when filling in the message text template to obtain an actual message
	 */
	public ASPLException (Localizer localizer, String localeTextTemplateID, String... params) {
		this.localizer = localizer;
		this.localeTextTemplateID = localeTextTemplateID;
		this.params = params;
	}

	/**
	 * Gets the exception message text, in the system's default locale
	 * 
	 * @return the exception message text, in the system's default locale
	 */
	public String getMessage () {
		return getMessage(Locale.getDefault());
	}

	/**
	 * Gets the exception message text, in the given locale
	 * 
	 * @param userLocale
	 *            the locale in which the message text is to be returned
	 * @return the exception message text, in the given locale
	 */
	public String getMessage (Locale userLocale) {
		return MyMessageFormat.format(localizer.getString(localeTextTemplateID, userLocale), params);
	}

}