package be.ASPL.presentationlayer;

import java.util.Locale;

import be.SIRAPRISE.util.MyMessageFormat;

/**
 * @author Erwin
 */
public class IntegerType extends PresentationType<IntegerValue> {

	/**
	 * @param val
	 *            -
	 * @return A GenericIntegerValue object holding the integer value
	 * @throws InvalidAttributeValueException
	 *             -
	 */
	public final static IntegerValue fromString (String val) throws InvalidAttributeValueException {
		try {
			return new IntegerValue(Integer.parseInt(val));
		} catch (NumberFormatException e) {
			throw new InvalidAttributeValueException(val, IntegerType.class.getSimpleName(), e.getMessage());
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#checkValidWaarValue(be.ASPL.presentationlayer.WaarValue, be.ASPL.presentationlayer.UserData)
	 */
	@Override
	public IntegerValue checkValidWaarValue (PresentationValue value, UserData userData) throws InvalidAttributeValueException {
		if (!(value instanceof IntegerValue)) {
			Locale locale = userData.getLocale();
			throw new InvalidAttributeValueException(value.toASPLString(userData), this.getClass().getSimpleName(), MyMessageFormat.format(I18N.getString(locale, "DynamicColumnList.InvalidValueClass"), new String[] { getClass().getName(), value.getClass().getName() })); //$NON-NLS-1$
		}
		return (IntegerValue) value;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#encode(byte[], be.ASPL.presentationlayer.UserData, java.util.Locale)
	 */
	@Override
	public IntegerValue encode (byte[] partBytes, UserData userData, Locale locale) {
		throw new UnsupportedOperationException();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.erwinsmout.waar.AttributeType#validate(java.lang.String)
	 */
	public IntegerValue encode (String val, UserData userData, Locale locale) throws InvalidAttributeValueException {
		return fromString(val);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.erwinsmout.ASPL.presentationlayer.PresentationType#getDefaultPresentationRenderingStyle()
	 */
	@Override
	public Renderer getDefaultPresentationRenderingStyle ( ) {
		return TextBoxRenderer.getInstance();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.erwinsmout.ASPL.presentationlayer.PresentationAttributeType#decode(be.erwinsmout.ASPL.WaarValue)
	 */
	public String getNakedStringRepresentation (PresentationValue presentationValue, OnLinePresentationFunction userFunction, PresentationField presentationField, Locale locale) {
		return Integer.toString(((IntegerValue) presentationValue).getInteger());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.erwinsmout.ASPL.presentationlayer.PresentationType#getPresentationRenderingStyle(java.lang.String)
	 */
	@Override
	public Renderer getPresentationRenderingStyle (String renderingStyleName) {
		if (renderingStyleName.equalsIgnoreCase("PASSWORD")) { //$NON-NLS-1$
			return PassWordRenderer.getInstance();
		}
		return TextBoxRenderer.getInstance();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#getWaarValue(boolean, java.util.Locale)
	 */
	@Override
	public PresentationValue getWaarValue (boolean b, Locale locale) {
		throw new UnsupportedOperationException();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#getWaarValue(int)
	 */
	@Override
	public PresentationValue getWaarValue (int v, Locale locale) {
		return new IntegerValue(v);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#getWaarValue(long)
	 */
	@Override
	public PresentationValue getWaarValue (long v, Locale locale) throws InvalidAttributeValueException {
		if (v <= Integer.MAX_VALUE && v >= Integer.MIN_VALUE) {
			return new IntegerValue((int) v);
		}
		throw new InvalidAttributeValueException(Long.toString(v), this.getClass().getName(), "[" + Integer.MIN_VALUE + "-" + Integer.MAX_VALUE + "]"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}
}
