package be.ASPL.presentationlayer;

import java.io.*;
import java.util.*;
import java.util.Map.Entry;

import javax.servlet.http.Part;

import be.SIRAPRISE.util.CollectionTransformer;
import be.SIRAPRISE.util.HTMLEntityRefs;
import be.SIRAPRISE.util.MyHashMap;
import be.SIRAPRISE.util.MyReadOnlyMap;

/**
 * @author Erwin
 */
public class Presentation {

	/**
	 * 
	 */
	private static final String __LINK__ = "__LINK__"; //$NON-NLS-1$

	/**
	 * hardcoded limit of 16M for file uploads
	 */
	private static final long MAX_PART_SIZE = 1 << 24;

	/**
	 * 
	 */
	private static final String PRESENTATIONDEF_SPECIAL_PREFIX_ACTION = "ACTION "; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final int PRESENTATIONDEF_SPECIAL_PREFIX_ACTION_LENGTH = PRESENTATIONDEF_SPECIAL_PREFIX_ACTION.length();

	/**
	 * 
	 */
	private static final String PROPERTIESSUFFIX = ".properties"; //$NON-NLS-1$

	/**
	 * 
	 */
	static final String PRESENTATIONSUFFIX = ".pres"; //$NON-NLS-1$

	/**
	 * @param presentationFields
	 *            -
	 * @return -
	 */
	protected static MyReadOnlyMap<String, PresentationField> getPresentationFieldsMap (PresentationField[] presentationFields) {
		return new MyReadOnlyMap<String, PresentationField>(MyHashMap.createMap(new CollectionTransformer<PresentationField, Map.Entry<String, PresentationField>>(PresentationFieldToMapEntryTransformer.getInstance()).transformToList(Arrays.asList(presentationFields))));
	}

	/**
	 * @param presentationName
	 *            -
	 * @param locale
	 *            -
	 * @param thisClass
	 *            -
	 * @return -
	 */
	protected static Properties getPresentationLabels (String presentationName, Locale locale, Class<?> thisClass) {
		final String languageUnspecifiedPresentationLabelsClassName = thisClass.getPackage().getName() + '.' + presentationName + "Labels"; //$NON-NLS-1$
		final String languageSpecificPresentationLabelsClassName = languageUnspecifiedPresentationLabelsClassName + '_' + locale.getLanguage();
		final String languageUnspecifiedApplicationLabelsClassName = thisClass.getPackage().getName() + '.' + "Labels"; //$NON-NLS-1$
		final String languageSpecificApplicationLabelsClassName = languageUnspecifiedApplicationLabelsClassName + '_' + locale.getLanguage();

		// load the Application-level labels for unspecified or unsupported languages
		Properties languageUnspecifiedApplicationLabels = new Properties();
		try (InputStream languageUnspecifiedApplicationLabelsInputStream = Architecture.getResourceInputStream(languageUnspecifiedApplicationLabelsClassName.replace('.', '/') + Presentation.PROPERTIESSUFFIX, thisClass);) {
			languageUnspecifiedApplicationLabels.load(languageUnspecifiedApplicationLabelsInputStream);
		} catch (IOException e) {
			languageUnspecifiedApplicationLabels.clear();
		} catch (ResourceNotFoundException e2) {
			languageUnspecifiedApplicationLabels.clear();
		}

		// load the presentation-level labels for unspecified or unsupported languages. The application-level labels for unspecified language are the default.
		Properties languageUnspecifiedPresentationLabels = new Properties(languageUnspecifiedApplicationLabels);
		try (InputStream languageUnspecifiedPresentationLabelsInputStream = Architecture.getResourceInputStream(languageUnspecifiedPresentationLabelsClassName.replace('.', '/') + Presentation.PROPERTIESSUFFIX, thisClass);) {
			languageUnspecifiedPresentationLabels.load(languageUnspecifiedPresentationLabelsInputStream);
		} catch (IOException e) {
			languageUnspecifiedPresentationLabels.clear();
		} catch (ResourceNotFoundException e2) {
			languageUnspecifiedPresentationLabels.clear();
		}

		// load the application-level labels for the specified locale. The presentation-level, and thus after that the application-level, labels without specified language are the default.
		Properties languageSpecificApplicationLabels = new Properties(languageUnspecifiedPresentationLabels);
		try (InputStream languageSpecificApplicationLabelsInputStream = Architecture.getResourceInputStream(languageSpecificApplicationLabelsClassName.replace('.', '/') + Presentation.PROPERTIESSUFFIX, thisClass);) {
			languageSpecificApplicationLabels.load(languageSpecificApplicationLabelsInputStream);
		} catch (IOException e) {
			languageSpecificApplicationLabels.clear();
		} catch (ResourceNotFoundException e2) {
			languageSpecificApplicationLabels.clear();
		}

		// load the presentation-level labels for the given locale
		Properties languageSpecificPresentationLabels = new Properties(languageSpecificApplicationLabels);
		try (InputStream languageSpecificPresentationLabelsInputStream = Architecture.getResourceInputStream(languageSpecificPresentationLabelsClassName.replace('.', '/') + Presentation.PROPERTIESSUFFIX, thisClass);) {
			languageSpecificPresentationLabels.load(languageSpecificPresentationLabelsInputStream);
		} catch (IOException e) {
			languageSpecificPresentationLabels.clear();
		} catch (ResourceNotFoundException e2) {
			languageSpecificPresentationLabels.clear();
		}

		return languageSpecificPresentationLabels;
	}

	/**
	 * The info message on the presentation
	 */
	private String message = ""; //$NON-NLS-1$

	/**
	 * The name of the presentation
	 */
	private String name;

	/**
	 * The ordered list of presentation-specific action buttons. The list contains the value for the value attribute of the html <INPUT> tag named "action".
	 */
	private LinkedList<String> presentationActions = new LinkedList<String>();

	/**
	 * The list of fields and their respective presentation attributes
	 */
	private HashMap<String, PresentationField> presentationFields = new HashMap<String, PresentationField>();

	/**
	 * The map of presentation attribute names and values
	 */
	private HashMap<String, PresentationValue> presentationValues = new HashMap<String, PresentationValue>();

	/**
	 * Creates the GenericPresentation object from its config file
	 * 
	 * @param presentationFunctionClass
	 *            The Class object of the presentation function. This is to allow access to related resources via that Class' ClassLoader (which is not necessarily the same as the architecutre's)
	 * @param name
	 *            The name of the presentation
	 * @param locale
	 *            -
	 * @throws ASPLException
	 *             -
	 * @deprecated - use the constructor that takes the prepared PresentationFields & PresentationActions for arguments
	 */
	@Deprecated
	public Presentation (Class<? extends OnLinePresentationFunction> presentationFunctionClass, String name, Locale locale) throws ASPLException {
		this.name = name;
		String w;

		PresentationField currentListField = null;
		LinkedList<PresentationField> listFieldStack = new LinkedList<PresentationField>();
		try (BufferedReader in = Architecture.getResourceReader(name + PRESENTATIONSUFFIX, presentationFunctionClass);) {
			while ((w = in.readLine()) != null) {
				w = w.trim();
				if (w.length() > 0 && !(w.startsWith("# "))) { //$NON-NLS-1$
					PresentationField presentationField;
					if (currentListField == null) {
						if (w.length() >= PRESENTATIONDEF_SPECIAL_PREFIX_ACTION_LENGTH && w.substring(0, PRESENTATIONDEF_SPECIAL_PREFIX_ACTION_LENGTH).equalsIgnoreCase(PRESENTATIONDEF_SPECIAL_PREFIX_ACTION)) {
							presentationActions.addLast(w.substring(PRESENTATIONDEF_SPECIAL_PREFIX_ACTION_LENGTH).trim());
						} else {
							presentationField = new PresentationField(w, presentationFunctionClass, locale);
							presentationFields.put(presentationField.getName(), presentationField);
							if (presentationField.getType() instanceof List || presentationField.getType() instanceof DynamicColumnList) {
								// if (currentListField != null) {
								// listFieldStack.addLast(currentListField);
								// }
								currentListField = presentationField;
								if (currentListField.getType() instanceof List) { // And thus not of dynamiccolumnlist
									currentListField.setTableRowHtml(name, presentationFunctionClass);
								}
							}
						}
					} else {
						// We're adding in a current list field
						if (w.equals("(")) { //$NON-NLS-1$
							// nothing to do, that's part of the old syntax. We have already set up the new current list field
						} else {
							if (w.equals(")")) { //$NON-NLS-1$
								if (listFieldStack.size() > 0) {
									currentListField = listFieldStack.removeLast();
								} else {
									currentListField = null;
								}
							} else {
								presentationField = new PresentationField(w, presentationFunctionClass, locale);
								currentListField.addColumn(presentationField);
								if (presentationField.getType() instanceof List || presentationField.getType() instanceof DynamicColumnList) {
									// if (currentListField != null) {
									listFieldStack.addLast(currentListField);
									// }
									currentListField = presentationField;
									if (currentListField.getType() instanceof List) { // And thus not of dynamiccolumnlist
										currentListField.setTableRowHtml(name, presentationFunctionClass);
									}
								}
							}
						}
					}
				}
			}
		} catch (IOException e) {
			presentationFields.clear();
			throw new PresentationDefinitionsIOException(e);
		} catch (ResourceNotFoundException e1) {
			throw new PresentationMissingException(name);
		}
	}

	/**
	 * @param name
	 *            The name of the presentation
	 * @param presentationFields
	 *            The list of presentationField Definitions that make up the presentation
	 * @param presentationActions
	 *            The list of user-defined actions on the presentation that can be performed by the user in addition to OK CANCEL HELP MAIN LOGOFF
	 * 
	 */
	public Presentation (String name, java.util.List<PresentationField> presentationFields, java.util.List<String> presentationActions) {
		this.name = name;
		this.presentationActions = new LinkedList<String>(presentationActions);
		for (PresentationField presentationField : presentationFields) {
			this.presentationFields.put(presentationField.getName(), presentationField);
			if (presentationField.getType() instanceof List) {
				presentationField.setTableRowHtmlNested(name, this.getClass());
			}
		}
	}

	/**
	 * @param locale
	 * 
	 */
	private void eraseInputFields (Locale locale) {
		Locale useLocale = locale == null ? Architecture.getDefaultLocale() : locale;
		for (Entry<String, PresentationField> me : presentationFields.entrySet()) {
			String attributeName = me.getKey();
			PresentationField presentationField = me.getValue();
			if (presentationField.isInputEnabled()) {
				// Fix the HTML feature that unchecked checkboxes are never transmitted.
				if (presentationField.getType().isTheBoolean()) {
					presentationValues.put(attributeName, presentationField.getType().getWaarValue(false, useLocale));
				} else {
					presentationValues.remove(attributeName);
				}
			}
		}
	}

	/**
	 * @param requestParameters
	 * @param userData
	 * @throws InvalidAttributeValueException
	 */
	private void setPresentationInputValuesFromLinkParameters (Map<String, String> requestParameters, UserData userData) throws InvalidAttributeValueException {

		for (Entry<String, String> rpme : requestParameters.entrySet()) {
			String requestParameterName = rpme.getKey();
			if (requestParameterName.startsWith(__LINK__)) {
				String sanitizedRequestParameterName = HTMLEntityRefs.V4_0.fromEntityRefsToUnicodeString(requestParameterName).substring(__LINK__.length());
				if (presentationFields.containsKey(sanitizedRequestParameterName)) {
					PresentationField presentationField = presentationFields.get(sanitizedRequestParameterName);
					// if (presentationField.isInputEnabled()) {
					String parameterValue = rpme.getValue();
					// Hack for the peculiar way in which HTML deals with checkboxes and boolean parameters
					Locale locale = userData == null ? Architecture.getDefaultLocale() : userData.getLocale();
					if (locale == null) {
						locale = Architecture.getDefaultLocale(); // is the case when starting the logon servlet
					}
					if (presentationField.getType() instanceof Logical) {
						if (parameterValue != null && parameterValue.length() > 0) {
							// parameterValue can still be "FALSE"
							// But beware : that introduces locale-dependency (what about "faux", "onwaar", ... ? And what about system-values, hardcoded in English when user chooses some other locale ?)
							//								this.presentationValues.put(sanitizedPresentationFieldName, presentationField.getType().encode(I18N.getString(locale, "Logical.True"), userData, locale)); //$NON-NLS-1$
							this.presentationValues.put(sanitizedRequestParameterName, presentationField.getType().encode(parameterValue, userData, locale));
						} else {
							this.presentationValues.put(sanitizedRequestParameterName, presentationField.getType().encode(I18N.getString(locale, "Logical.False"), userData, locale)); //$NON-NLS-1$
						}
					} else {
						if (parameterValue != null && parameterValue.length() > 0) {
							this.presentationValues.put(sanitizedRequestParameterName, presentationField.getType().encode(HTMLEntityRefs.V4_0.fromEntityRefsToUnicodeString(parameterValue), userData, locale));
						}
					}
					// }
				}
			}
		}
	}

	/**
	 * @param requestParameters
	 * @param requestMultiPartConfigParts
	 * @param userData
	 * @throws InvalidAttributeValueException
	 * @throws PartSizeExceedsLimitsException
	 * @throws IOException
	 * @throws IncompletePartException
	 */
	private void setPresentationInputValuesFromReqParameters (Map<String, String> requestParameters, Collection<Part> requestMultiPartConfigParts, UserData userData) throws InvalidAttributeValueException, PartSizeExceedsLimitsException, IOException, IncompletePartException {

		for (Entry<String, String> rpme : requestParameters.entrySet()) {

			String parameterName = rpme.getKey();
			String sanitizedPresentationFieldName = HTMLEntityRefs.V4_0.fromEntityRefsToUnicodeString(parameterName);
			if (presentationFields.containsKey(sanitizedPresentationFieldName)) {
				PresentationField presentationField = presentationFields.get(sanitizedPresentationFieldName);
				if (presentationField.isInputEnabled()) {

					String parameterValue = rpme.getValue();
					Locale locale = userData == null ? Architecture.getDefaultLocale() : userData.getLocale();
					if (locale == null) {
						locale = Architecture.getDefaultLocale(); // is the case when starting the logon servlet
					}
					// Hack for the peculiar way in which HTML deals with checkboxes and boolean parameters
					if (presentationField.getType() instanceof Logical) {
						if (parameterValue != null && parameterValue.length() > 0) {
							// parameterValue can still be "FALSE"
							// But beware : that introduces locale-dependency (what about "faux", "onwaar", ... ? And what about system-values, hardcoded in English when user chooses some other locale ?)
							//							this.presentationValues.put(sanitizedPresentationFieldName, presentationField.getType().encode(I18N.getString(locale, "Logical.True"), userData, locale)); //$NON-NLS-1$
							this.presentationValues.put(sanitizedPresentationFieldName, presentationField.getType().encode(parameterValue, userData, locale));
						} else {
							this.presentationValues.put(sanitizedPresentationFieldName, presentationField.getType().encode(I18N.getString(locale, "Logical.False"), userData, locale)); //$NON-NLS-1$
						}
					} else {
						if (parameterValue != null && parameterValue.length() > 0) {
							this.presentationValues.put(sanitizedPresentationFieldName, presentationField.getType().encode(HTMLEntityRefs.V4_0.fromEntityRefsToUnicodeString(parameterValue), userData, locale));
						}
					}
				}
			}
		}

		for (Part part : requestMultiPartConfigParts) {
			String sanitizedPartName = HTMLEntityRefs.V4_0.fromEntityRefsToUnicodeString(part.getName());
			if (presentationFields.containsKey(sanitizedPartName) && !presentationValues.containsKey(sanitizedPartName)) {
				PresentationField presentationField = presentationFields.get(sanitizedPartName);
				if (presentationField.isInputEnabled()) {

					Locale locale = userData == null ? Architecture.getDefaultLocale() : userData.getLocale();
					if (locale == null) {
						locale = Architecture.getDefaultLocale(); // is the case when starting the logon servlet
					}
					// We don't need to Hack for the peculiar way in which HTML deals with checkboxes and boolean parameters, because FileUploads are assumed not to represent a boolean
					InputStream parameterInputStream = part.getInputStream();
					final long partSize = part.getSize();
					if (parameterInputStream != null && partSize > 0) {
						if (partSize < MAX_PART_SIZE) {
							byte[] partBytes = new byte[(int) partSize];
							int bytesRead = parameterInputStream.read(partBytes);
							if (bytesRead != partSize) {
								throw new IncompletePartException(bytesRead, partSize);
							}
							this.presentationValues.put(sanitizedPartName, presentationField.getType().encode(partBytes, userData, locale));
						} else {
							throw new PartSizeExceedsLimitsException(partSize);
						}
					}
				}
			}
		}
	}

	/**
	 * Gets the labels defined for this presentation, for the specified locale
	 * 
	 * @param locale
	 * 
	 * @return
	 */
	Properties getLabels (Locale locale) {
		return getPresentationLabels(this.getName(), locale, this.getClass());
	}

	/**
	 * Gets The ordered list of presentation-specific action buttons. The list contains the value for the value attribute of the html <INPUT> tag named "action".
	 * 
	 * @return The ordered list of presentation-specific action buttons. The list contains the value for the value attribute of the html <INPUT> tag named "action".
	 */
	LinkedList<String> getPresentationActions ( ) {
		return presentationActions;
	}

	/**
	 * Sets the presentation message field
	 * 
	 * @param message
	 *            the message to be displayed
	 */
	void setMessage (String message) {
		this.message = message;
	}

	/**
	 * @param requestParameters
	 * @param requestMultiPartConfigParts
	 * @param userData
	 * @throws MandatoryFieldMissingException
	 * @throws InvalidAttributeValueException
	 * @throws PartSizeExceedsLimitsException
	 * @throws IOException
	 * @throws IncompletePartException
	 */
	void setPresentationValuesIgnoringDefaults (Map<String, String> requestParameters, Collection<Part> requestMultiPartConfigParts, UserData userData) throws MandatoryFieldMissingException, InvalidAttributeValueException, PartSizeExceedsLimitsException, IOException, IncompletePartException {
		// this.presentationValues = getPresentationWaarValueMap(req, userData, false);
		final Locale useUserData = userData == null ? Architecture.getDefaultLocale() : userData.getLocale();
		eraseInputFields(useUserData);
		setPresentationInputValuesFromReqParameters(requestParameters, requestMultiPartConfigParts, userData);

		for (PresentationField presentationField : presentationFields.values()) {
			String attributeName = presentationField.getName();
			if (presentationField.isInputEnabled() && presentationField.isMandatory() && presentationValues.get(attributeName) == null) {
				throw new MandatoryFieldMissingException(attributeName);
			}
		}
	}

	/**
	 * @param requestParameters
	 * @param requestMultiPartConfigParts
	 * @param userData
	 * @param locallyConfiguredPresentationDefaults
	 * @throws InvalidAttributeValueException
	 * @throws PartSizeExceedsLimitsException
	 * @throws IOException
	 * @throws IncompletePartException
	 */
	void setPresentationValuesIncludingDefaults (Map<String, String> requestParameters, Collection<Part> requestMultiPartConfigParts, UserData userData, Properties locallyConfiguredPresentationDefaults) throws InvalidAttributeValueException, PartSizeExceedsLimitsException, IOException, IncompletePartException {
		eraseInputFields(userData == null ? Architecture.getDefaultLocale() : userData.getLocale());
		setPresentationInputValuesFromReqParameters(requestParameters, requestMultiPartConfigParts, userData);
		setPresentationInputValuesFromLinkParameters(requestParameters, userData);

		// get all the default values off the given locally configured presentation defaults
		for (Entry<Object, Object> locallyConfiguredDefaultEntry : locallyConfiguredPresentationDefaults.entrySet()) {
			final String presentationFieldNameWithLocallyConfiguredDefault = (String) locallyConfiguredDefaultEntry.getKey();
			if (getPresentationFields().containsKey(presentationFieldNameWithLocallyConfiguredDefault)) {
				if (!presentationValues.containsKey(presentationFieldNameWithLocallyConfiguredDefault)) {
					try {
						presentationValues.put(presentationFieldNameWithLocallyConfiguredDefault, presentationFields.get(presentationFieldNameWithLocallyConfiguredDefault).getType().encode((String) locallyConfiguredDefaultEntry.getValue(), userData, Architecture.getDefaultLocale()));
					} catch (InvalidAttributeValueException e) {
						//if locally configured defaults contain invalid values, skip them after logging a message
					}
				}
			}
		}

		// get all the default values off the presentationfielddefinitions
		for (PresentationField presentationField : getPresentationFields().values()) {
			if (presentationField.getDefaultValue() != null) {
				if (!presentationValues.containsKey(presentationField.getName())) {
					presentationValues.put(presentationField.getName(), presentationField.getDefaultValue());
				}
			}
		}
	}

	/**
	 * Gets the Presentation message field
	 * 
	 * @return the Presentation message field
	 */
	public final String getMessage ( ) {
		return message;
	}

	/**
	 * Gets the name of the current presentation
	 * 
	 * @return The name of the current presentation
	 */
	public final String getName ( ) {
		return name;
	}

	/**
	 * @param presentationFieldName
	 *            -
	 * @return the PresentationField object
	 * @throws ASPLException
	 *             -
	 */
	public final PresentationField getPresentationField (String presentationFieldName) throws ASPLException {
		if (presentationFields.containsKey(presentationFieldName)) {
			return presentationFields.get(presentationFieldName);
		} else {
			throw new PresentationFieldMissingException(presentationFieldName, name);
		}
	}

	/**
	 * Gets the list of fields and their respective presentation attributes
	 * 
	 * @return The list of fields and their respective presentation attributes
	 */
	public final HashMap<String, PresentationField> getPresentationFields ( ) {
		return presentationFields;
	}

	/**
	 * Gets the map of presentation attribute names and values
	 * 
	 * @return The map of presentation attribute names and values
	 */
	public final HashMap<String, PresentationValue> getPresentationValues ( ) {
		return presentationValues;
	}

	/**
	 * @param presentationFieldName
	 *            -
	 * @param mode
	 *            -
	 * @throws ASPLException
	 *             -
	 */
	public final void setPresentationFieldInOutMode (String presentationFieldName, InOutMode mode) throws ASPLException {
		getPresentationField(presentationFieldName).setInOutMode(mode);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString ( ) {
		return name + presentationFields;
	}
}
