package be.ASPL.webclient;

import java.util.*;

import be.ASPL.presentationlayer.*;
import be.SIRAPRISE.util.MyString;

/**
 * @author Erwin
 */
public class UsersManage extends UsersFunction {

	/**
	 * Comment for <code>leave</code>
	 */
	private boolean leave = false;

	/**
	 * @throws ASPLException
	 */
	private void processSelectedUser ( ) throws ASPLException {
		PresentationValue wv_userID = getPresentationValue("USERID"); //$NON-NLS-1$
		UserProperties userProperties = Architecture.getUserProperties(getApplicationName(), ((StringValue) wv_userID).getString().trim());

		// switch from 'select relvar' presentation to 'manage some particular relvar's details' presentation
		setPresentation(new UsersManageDetailsPresentation());
		setPresentationValue("USERID", wv_userID); //$NON-NLS-1$
		setPresentationValue("DISABLED", new LogicalValue(userProperties.isDisabled())); //$NON-NLS-1$
		setPresentationValue("PWCHANGEREQUIRED", new LogicalValue(userProperties.isPasswordToBeChangedAtNextLogon())); //$NON-NLS-1$
		setPresentationValue("LANGUAGE", userProperties.getLanguage()); //$NON-NLS-1$
		setPresentationValue("ROLES", new StringValue(MyString.asCSV(userProperties.getRoles()).toLowerCase())); //$NON-NLS-1$

		ListValue wv_pwFailures = new ListValue("PWFAILURES"); //$NON-NLS-1$
		Iterator<Long> i_suspiciousLogonAttemptTimes = userProperties.getSuspiciousLogonAttemptTimes().iterator();
		while (i_suspiciousLogonAttemptTimes.hasNext()) {
			HashMap<String, PresentationValue> m = new HashMap<String, PresentationValue>();
			m.put("PASSWORDFAILURE", new TimeStampValue(i_suspiciousLogonAttemptTimes.next().longValue())); //$NON-NLS-1$
			wv_pwFailures.append(m);
		}
		setPresentationValue("PWFAILURES", wv_pwFailures); //$NON-NLS-1$

		saveInFunctionContext("USERPROPERTIES", userProperties); //$NON-NLS-1$
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doInitialLogic()
	 */
	public void doInitialLogic ( ) throws ASPLException {
		if (getPresentation().getName().equalsIgnoreCase("USERSSELECT")) {//$NON-NLS-1$
			if (getPresentationValue("USERID") != null) { //$NON-NLS-1$
				processSelectedUser();
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doPresentationLogic()
	 */
	public void doPresentationLogic ( ) throws ASPLException {
		switch (getPresentation().getName().toUpperCase()) {
			case "USERSSELECT": { //$NON-NLS-1$
				// selection criteria entered, inquire catalog and switch to managedetails presentation
				processSelectedUser();
				break;
			}
			case "USERSMANAGEDETAILS": { //$NON-NLS-1$
				final String userID = ((StringValue) getPresentationValue("USERID")).getString().trim(); //$NON-NLS-1$
				final String language = ((StringValue) getPresentationValue("LANGUAGE")).getString(); //$NON-NLS-1$
				final String rolesCSV = ((StringValue) getPresentationValue("ROLES")).getString().trim().toLowerCase(); //$NON-NLS-1$

				if (getPresentationValue("PASSWORD") != null) { //$NON-NLS-1$
					Architecture.removeWaarUser(getApplicationName(), userID);
					Architecture.newWaarUser(getApplicationName(), userID, ((StringValue) getPresentationValue("PASSWORD")).getString(), language, rolesCSV); //$NON-NLS-1$
				} else {
					Architecture.checkAndSetLanguagePlusRoles(getApplicationName(), userID, language, rolesCSV);
				}
				leave = true;
				break;
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#getInitialPresentationClass()
	 */
	public Class<? extends Presentation> getInitialPresentationClass ( ) {
		if (getUserRoles().contains(Architecture.ASPLADMINISTRATOR_ROLENAME)) {
			return UsersSelectPresentation.class;
		} else {
			return UsersVoidPresentation.class;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#getPresentationFunctionDescription()
	 */
	public String getPresentationFunctionDescription () {
		return "View a WAAR User's Details"; //$NON-NLS-1$
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#leaveFunctionOnExit()
	 */
	public boolean leaveFunctionOnExit ( ) {
		return leave;
	}
}
