/**
 * 
 */
package be.SIRAPRISE.birdwatcher;

import java.util.Set;

import be.ASPL.presentationlayer.ASPLApplicationException;
import be.ASPL.presentationlayer.DateValue;
import be.ASPL.presentationlayer.IntegerValue;
import be.ASPL.presentationlayer.StringValue;
import be.ASPL.presentationlayer.LogicalValue;
import be.SIRAPRISE.client.ConnectionClosedException;
import be.SIRAPRISE.client.DBConnection;
import be.SIRAPRISE.client.RelationValueSelectorFactory;
import be.SIRAPRISE.client.TransactionMode;
import be.SIRAPRISE.messages.ErrorMessageException;
import be.SIRAPRISE.typeimplementations.TYPENAMES;
import be.SIRAPRISE.util.ISODateFormat;
import be.SIRAPRISE.util.MyString;

/**
 * @author Erwin
 * 
 */
public abstract class BWSpotsFunction extends BWWebClientFunction {

	/**
	 * @return
	 * @throws ASPLApplicationException
	 */
	be.SIRAPRISE.typeimplementations.Relation getMatchingSpots ( ) throws ASPLApplicationException {

		StringValue name = (StringValue) getPresentationValue(ATTRIBUTENAMES.SPECIESNAME);
		String unknownGenderSpotsQuery = "EXTEND(" + RELVARNAMES.SPOTS + ",(GENDERKNOWN(BOOLEAN(FALSE))" + ATTRIBUTENAMES.GENDER + "(STRING( ))))"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		String knownGenderSpotsQuery = "TRANSFORM(RENAME(" + RELVARNAMES.GSPOTS + ",(" + ATTRIBUTENAMES.GENDER + ",G)),(" + ATTRIBUTENAMES.SPOTCOUNT + "," + ATTRIBUTENAMES.WALKDATE + "," + ATTRIBUTENAMES.SPECIESNAME + ",GENDERKNOWN(BOOLEAN(TRUE))," + ATTRIBUTENAMES.GENDER + "(THE_STRING(G))))"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$ //$NON-NLS-7$
		String spotsQuery;
		if (((LogicalValue) getPresentationValue("SELECTBYKNOWNORUNKNOWNGENDER")).getValue()) { //$NON-NLS-1$
			if (((LogicalValue) getPresentationValue("GENDERKNOWN")).getValue()) { //$NON-NLS-1$
				StringValue wv_gender = (StringValue) getPresentationValue(ATTRIBUTENAMES.GENDER);
				if (wv_gender.getString().equalsIgnoreCase(" ")) { //$NON-NLS-1$
					spotsQuery = knownGenderSpotsQuery;
				} else {
					spotsQuery = "RESTRICT(" + knownGenderSpotsQuery + ",EQ(GENDER,STRING(" + wv_gender.getString() + ")))"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
				}
			} else {
				spotsQuery = unknownGenderSpotsQuery;
			}
		} else {
			spotsQuery = "UNION(" + unknownGenderSpotsQuery + "," + knownGenderSpotsQuery + ")"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$ 
		}
		if (name != null) {
			Set<String> names = MyString.getNames(name.getString(), ',');
			String nameValueSelector = RelationValueSelectorFactory.getUnaryRelationValueSelectorExpressionText(names, ATTRIBUTENAMES.SPECIESNAME, TYPENAMES.STRING);
			spotsQuery = "JOIN(" + spotsQuery + "," + nameValueSelector + ")"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}

		DateValue walkDate = (DateValue) getPresentationValue(ATTRIBUTENAMES.WALKDATE);
		if (walkDate != null) {
			spotsQuery = "RESTRICT(" + spotsQuery + ",EQ(DATE(" + ISODateFormat.formatISO(walkDate.getDate()) + ")," + ATTRIBUTENAMES.WALKDATE + "))"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$
		} else {
			DateValue fromDate = (DateValue) getPresentationValue("FROMDATE"); //$NON-NLS-1$
			if (fromDate != null) {
				spotsQuery = "RESTRICT(" + spotsQuery + ",NOT(GT(DATE(" + ISODateFormat.formatISO(fromDate.getDate()) + ")," + ATTRIBUTENAMES.WALKDATE + ")))"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$
			}

			DateValue toDate = (DateValue) getPresentationValue("TODATE"); //$NON-NLS-1$
			if (toDate != null) {
				spotsQuery = "RESTRICT(" + spotsQuery + ",GT(DATE(" + ISODateFormat.formatISO(toDate.getDate()) + ")," + ATTRIBUTENAMES.WALKDATE + "))"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$
			}
		}

		IntegerValue fromCount = (IntegerValue) getPresentationValue("FROMCOUNT"); //$NON-NLS-1$
		if (fromCount != null) {
			spotsQuery = "RESTRICT(" + spotsQuery + ",NOT(GT(INT(" + fromCount + ")," + ATTRIBUTENAMES.SPOTCOUNT + ")))"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$
		}

		IntegerValue toCount = (IntegerValue) getPresentationValue("TOCOUNT"); //$NON-NLS-1$
		if (toCount != null) {
			spotsQuery = "RESTRICT(" + spotsQuery + ",GT(INT(" + toCount + ")," + ATTRIBUTENAMES.SPOTCOUNT + "))"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$
		}

		StringValue language = (StringValue) getPresentationValue(ATTRIBUTENAMES.LANGUAGE);
		if (language != null) {
			spotsQuery = "EXTEND(LEFTJOIN(" + spotsQuery + ",PROJECT(RESTRICT(" + RELVARNAMES.NLSPECIESNAME + ",EQ(" + ATTRIBUTENAMES.LANGUAGE + ",LANGUAGE(ISO2(" + language.getString() + ")))),(" + ATTRIBUTENAMES.SPECIESNAME + "," + ATTRIBUTENAMES.NLSPECIESNAME + ")),(" + ATTRIBUTENAMES.NLSPECIESNAME + "(STRING( )))),(NLANNOTATEDNAME(CONCAT(" + ATTRIBUTENAMES.SPECIESNAME + ",CONCAT(STRING( -),CONCAT(" + ATTRIBUTENAMES.NLSPECIESNAME + ",STRING(-)))))))"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$//$NON-NLS-5$//$NON-NLS-6$//$NON-NLS-7$ //$NON-NLS-8$ //$NON-NLS-9$ //$NON-NLS-10$
		}

		
		try (DBConnection dbc = getDBConnection();) {
			return dbc.startTransaction(getUserData().getUserID(), true, "", new byte[] {}, TransactionMode.READONLY).execQueryAndEndTransaction(spotsQuery); //$NON-NLS-1$
		} catch (ConnectionClosedException e) {
			log(e);
			throw new ASPLApplicationException(e);
		} catch (ErrorMessageException e) {
			log(e);
			throw new ASPLApplicationException(e);
		}
	}
}
