package be.SIRAPRISE.birdwatcher;

import java.util.Locale;

import be.ASPL.presentationlayer.*;
import be.SIRAPRISE.client.*;
import be.SIRAPRISE.client.jsba.ClassDoesNotImplementDBObjectException;
import be.SIRAPRISE.client.jsba.ConstructorMissingException;
import be.SIRAPRISE.client.jsba.DBObjectFactory;
import be.SIRAPRISE.client.jsba.DmlCommand;
import be.SIRAPRISE.messages.ErrorMessageException;
import be.SIRAPRISE.typeimplementations.Relation;

/**
 * The function to produce an overview of defined attributes
 * 
 * @author Erwin
 */
@InitialPresentation(initialPresentationClass = SpotsOverviewPresentation.class)
public class SpotsDelete extends BWSpotsFunction {

	/**
	 * 
	 */
	private boolean leave;

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doInitialLogic()
	 */
	public void doInitialLogic ( ) throws ASPLException {
		setDefaultLanguageFromUserContext();
		if (getPresentationValue(ATTRIBUTENAMES.SPECIESNAME) != null || getPresentationValue(ATTRIBUTENAMES.WALKDATE) != null || getPresentationValue("FROMDATE") != null || getPresentationValue("TODATE") != null || getPresentationValue("FROMCOUNT") != null || getPresentationValue("TOCOUNT") != null) { //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
			processSelectionPresentation();
		}
	}

	/**
	 * @throws ASPLException
	 */
	private void processSelectionPresentation ( ) throws ASPLException {
		Relation r = getMatchingSpots();
		if (r.size() == 1) {
			try {
				Spot spot = DBObjectFactory.getObjectCollectionIgnoringMissingSetters(r, Spot.class).iterator().next();
				setPresentation(new SpotsDeleteConfirmPresentation());
				setPresentationValue(ATTRIBUTENAMES.WALKDATE, new DateValue(spot.getWalkDate()));
				setPresentationValue(ATTRIBUTENAMES.SPECIESNAME, spot.getSpeciesName());
				setPresentationValue(ATTRIBUTENAMES.SPOTCOUNT, spot.getSpotCount());
				setPresentationValue(ATTRIBUTENAMES.GENDER, spot.getGender());

				saveInFunctionContext(RELVARNAMES.SPOTS, spot);
			} catch (ConstructorMissingException e) {
				throw new RuntimeException(e);
			} catch (ClassDoesNotImplementDBObjectException e) {
				throw new RuntimeException(e);
			}
		} else {
			Locale locale = getUserLocale();
			setPresentationValue("SPOTSLIST", ASPLListFactory.getASPLListValue(getPresentation().getPresentationField("SPOTSLIST"), r, getUserData(), locale)); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doPresentationLogic()
	 */
	public void doPresentationLogic ( ) throws ASPLException {
		if (getPresentation().getName().equalsIgnoreCase("SpotsOverview")) { //$NON-NLS-1$
			processSelectionPresentation();
		} else {
			Spot spot = (Spot) getFromFunctionContextMandatory(RELVARNAMES.SPOTS);

			
			try (DBConnection dbc = getDBConnection();) {
				DmlCommand.execDmlCommandAndEndTransaction(dbc.startTransaction(getUserData().getUserID(), true, "", new byte[] {}, TransactionMode.UPDATE), spot.dbDeleteCommand(spot.getGender() == ' ' ? RELVARNAMES.SPOTS : RELVARNAMES.GSPOTS)); //$NON-NLS-1$
				leave = true;
			} catch (ConnectionClosedException e) {
				log(e);
				throw new ASPLApplicationException(e);
			} catch (ErrorMessageException e) {
				log(e);
				throw new ASPLApplicationException(e);
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#getPresentationFunctionDescription()
	 */
	public String getPresentationFunctionDescription () {
		return I18N.getString(getUserLocale(), "SpotsDelete.Text"); //$NON-NLS-1$
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#leaveFunctionOnExit()
	 */
	public boolean leaveFunctionOnExit ( ) {
		return leave;
	}
}