/**
 * 
 */
package be.SIRAPRISE.tests.officialtestset;

import static org.junit.Assert.fail;

import java.io.IOException;
import java.util.Properties;

import org.junit.*;

import be.SIRAPRISE.client.*;
import be.SIRAPRISE.client.NAMES.*;
import be.SIRAPRISE.client.jsba.*;
import be.SIRAPRISE.messages.CommunicationProtocolException;
import be.SIRAPRISE.messages.ErrorMessageException;
import be.SIRAPRISE.typeimplementations.TYPENAMES;
import be.SIRAPRISE.util.MyCommandLineParameters;

/**
 * @author Erwin
 * 
 */
public final class EngineVolumeSpeedTest {

	/**
	 * 
	 */
	private static final String _CUND = "C_"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String _HUND = "H_"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String VAIT_L1 = "VAIT_L1"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String VAIT_L2 = "VAIT_L2"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String VAIT_R0 = "VAIT_R0"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String VAIT_R0_C = _CUND + VAIT_R0;

	/**
	 * 
	 */
	private static final String VAIT_R0_H = _HUND + VAIT_R0;

	/**
	 * 
	 */
	private static final String VAIT_R1 = "VAIT_TR1"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String VAIT_R1_C = _CUND + VAIT_R1;

	/**
	 * 
	 */
	private static final String VAIT_R1_H = _HUND + VAIT_R1;

	/**
	 * 
	 */
	private static final String VAIT_R12 = "VAIT_TR12"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String VAIT_R12_C = _CUND + VAIT_R12;

	/**
	 * 
	 */
	private static final String VAIT_R12_H = _HUND + VAIT_R12;

	/**
	 * 
	 */
	private static final String VAIT_R2 = "VAIT_TR2"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String VAIT_R2_C = _CUND + VAIT_R2;

	/**
	 * 
	 */
	private static final String VAIT_R2_H = _HUND + VAIT_R2;

	/**
	 * 
	 */
	private static final String VAIT_SPDB = "VAIT.SPDB"; //$NON-NLS-1$

	/**
	 * Accepts commands from the console and runs them through the server
	 * 
	 * @param args
	 *            No arguments are used
	 */
	public static void main (String args[]) {
		final EngineVolumeSpeedTest engineVolumeStressTest = new EngineVolumeSpeedTest();
		engineVolumeStressTest.setProperties(MyCommandLineParameters.getCommandLineParameters(args, null, EngineVolumeSpeedTestProperties.getInstance()));
		engineVolumeStressTest.test();
	}

	/**
	 * 
	 */
	private int commitSize;

	/**
	 * 
	 */
	private String hostName;

	/**
	 * 
	 */
	private int monitorport;

	/**
	 * 
	 */
	private int port;

	/**
	 * 
	 */
	private int totalBatchSize;

	/**
	 * Constructor for the JUnit framework
	 */
	public EngineVolumeSpeedTest ( ) {

	}

	/**
	 * @param properties
	 */
	private void setProperties (Properties properties) {
		hostName = properties.getProperty(DBConnectionProperties.HOST, DBConnectionProperties.HOSTDEFAULT);
		port = Integer.parseInt(properties.getProperty(DBConnectionProperties.PORT, DBConnectionProperties.PORTDEFAULT));
		monitorport = Integer.parseInt(properties.getProperty(DBConnectionProperties.MONITORPORT, DBConnectionProperties.MONITORPORTDEFAULT));
		totalBatchSize = Integer.parseInt(properties.getProperty(EngineVolumeSpeedTestProperties.TOTALBATCHSIZE, EngineVolumeSpeedTestProperties.TOTALBATCHSIZEDEFAULT));
		commitSize = Integer.parseInt(properties.getProperty(EngineVolumeSpeedTestProperties.COMMITSIZE, EngineVolumeSpeedTestProperties.COMMITSIZEDEFAULT));
	}

	/**
	 * 
	 */
	@Before
	public void before ( ) {
		setProperties(EngineVolumeSpeedTestProperties.getInstance());
	}

	/**
	 * 
	 */
	@SuppressWarnings("resource")
	@Test
	public void test ( ) {

		// Get the connection
		OneShotDBConnection dbc = null;
		do {
			try {
				// get the DB connection
				dbc = new OneShotDBConnection(new SiraPriseServer(hostName, port, monitorport), "", null); //$NON-NLS-1$
			} catch (DBException e1) {
				if (e1.getErrorCode() == 454) { // max connections exceeded
					try {
						Thread.sleep(200);
					} catch (InterruptedException e) {

					}
				} else {
					fail(e1.getClass().getName() + ' ' + e1.getMessage());
				}
			} catch (IOException e) {
				fail(e.getClass().getName() + ' ' + e.getMessage());
			} catch (CommunicationProtocolException e) {
				fail(e.getClass().getName() + ' ' + e.getMessage());
			}
		} while (dbc == null);

		// Start a transaction and start processing
		try {
			final Attribute a1 = new Attribute(VAIT_L1, TYPENAMES.LONG, -1);
			final Attribute a2 = new Attribute(VAIT_L2, TYPENAMES.LONG, -1);
			final Relvar r0 = new Relvar(VAIT_R0, "VAIT_L1 is \u00A7vait_l1\u00A7 and VAIT_L2 is \u00A7vait_l2\u00A7."); //$NON-NLS-1$
			final Relvar r1 = new Relvar(VAIT_R1, "VAIT_L1 is \u00A7vait_l1\u00A7 and VAIT_L2 is \u00A7vait_l2\u00A7."); //$NON-NLS-1$
			final Relvar r2 = new Relvar(VAIT_R2, "VAIT_L1 is \u00A7vait_l1\u00A7 and VAIT_L2 is \u00A7vait_l2\u00A7."); //$NON-NLS-1$
			final Relvar r12 = new Relvar(VAIT_R12, "VAIT_L1 is \u00A7vait_l1\u00A7 and VAIT_L2 is \u00A7vait_l2\u00A7."); //$NON-NLS-1$
			final Key k0 = new Key(VAIT_R0, 21123456);
			final Key k1 = new Key(VAIT_R1, 22123456);
			final Key k2 = new Key(VAIT_R2, 23123456);
			final Key k12 = new Key(VAIT_R12, 24123456);
			final RelvarAttribute ra0_l1 = new RelvarAttribute(VAIT_R0, VAIT_L1);
			final RelvarAttribute ra0_l2 = new RelvarAttribute(VAIT_R0, VAIT_L2);
			final RelvarAttribute ra1_l1 = new RelvarAttribute(VAIT_R1, VAIT_L1);
			final RelvarAttribute ra1_l2 = new RelvarAttribute(VAIT_R1, VAIT_L2);
			final RelvarAttribute ra2_l1 = new RelvarAttribute(VAIT_R2, VAIT_L1);
			final RelvarAttribute ra2_l2 = new RelvarAttribute(VAIT_R2, VAIT_L2);
			final RelvarAttribute ra12_l1 = new RelvarAttribute(VAIT_R12, VAIT_L1);
			final RelvarAttribute ra12_l2 = new RelvarAttribute(VAIT_R12, VAIT_L2);
			final KeyAttribute ka0 = new KeyAttribute(21123456, VAIT_L1);
			final KeyAttribute ka1 = new KeyAttribute(22123456, VAIT_L1);
			final KeyAttribute ka2 = new KeyAttribute(23123456, VAIT_L1);
			final KeyAttribute ka12 = new KeyAttribute(24123456, VAIT_L1);

			final DbmsFile f = new DbmsFile(VAIT_SPDB, 8192);
			final HashedRecordSpace hrs0 = new HashedRecordSpace(VAIT_SPDB, 1, 1668, 9, 15, 4);
			final HashedRecordSpace hrs1 = new HashedRecordSpace(VAIT_SPDB, 2, 1668, 9, 15, 4);
			final HashedRecordSpace hrs2 = new HashedRecordSpace(VAIT_SPDB, 3, 1668, 9, 15, 4);
			final HashedRecordSpace hrs12 = new HashedRecordSpace(VAIT_SPDB, 4, 1668, 9, 15, 4);
			final ClusteredRecordSpace crs0 = new ClusteredRecordSpace(VAIT_SPDB, 11, 1668, 9);
			final ClusteredRecordSpace crs1 = new ClusteredRecordSpace(VAIT_SPDB, 12, 1668, 9);
			final ClusteredRecordSpace crs2 = new ClusteredRecordSpace(VAIT_SPDB, 13, 1668, 9);
			final ClusteredRecordSpace crs12 = new ClusteredRecordSpace(VAIT_SPDB, 14, 1668, 9);
			final ClusteredRecordSpaceLocAttrs crsla1 = new ClusteredRecordSpaceLocAttrs(VAIT_SPDB, 11, 1, VAIT_L1);
			final ClusteredRecordSpaceLocAttrs crsla2 = new ClusteredRecordSpaceLocAttrs(VAIT_SPDB, 12, 1, VAIT_L1);
			final ClusteredRecordSpaceLocAttrs crsla3 = new ClusteredRecordSpaceLocAttrs(VAIT_SPDB, 13, 1, VAIT_L1);
			final ClusteredRecordSpaceLocAttrs crsla4 = new ClusteredRecordSpaceLocAttrs(VAIT_SPDB, 14, 1, VAIT_L1);

			final RecordType rt0h = new RecordType(VAIT_R0, VAIT_R0_H, 60, VAIT_SPDB, 1);
			final RecordType rt0c = new RecordType(VAIT_R0, VAIT_R0_C, 60, VAIT_SPDB, 11);
			final RecordType rt1h = new RecordType(VAIT_R1, VAIT_R1_H, 60, VAIT_SPDB, 2);
			final RecordType rt1c = new RecordType(VAIT_R1, VAIT_R1_C, 60, VAIT_SPDB, 12);
			final RecordType rt2h = new RecordType(VAIT_R2, VAIT_R2_H, 60, VAIT_SPDB, 3);
			final RecordType rt2c = new RecordType(VAIT_R2, VAIT_R2_C, 60, VAIT_SPDB, 13);
			final RecordType rt12h = new RecordType(VAIT_R12, VAIT_R12_H, 60, VAIT_SPDB, 4);
			final RecordType rt12c = new RecordType(VAIT_R12, VAIT_R12_C, 60, VAIT_SPDB, 14);

			final RecordAttribute ra0h = new RecordAttribute(VAIT_R0, VAIT_R0_H, 60, VAIT_L1);
			final RecordAttribute ra0c = new RecordAttribute(VAIT_R0, VAIT_R0_C, 60, VAIT_L1);
			final RecordAttribute ra1h = new RecordAttribute(VAIT_R1, VAIT_R1_H, 60, VAIT_L1);
			final RecordAttribute ra1c = new RecordAttribute(VAIT_R1, VAIT_R1_C, 60, VAIT_L1);
			final RecordAttribute ra2h = new RecordAttribute(VAIT_R2, VAIT_R2_H, 60, VAIT_L1);
			final RecordAttribute ra2c = new RecordAttribute(VAIT_R2, VAIT_R2_C, 60, VAIT_L1);
			final RecordAttribute ra12h = new RecordAttribute(VAIT_R12, VAIT_R12_H, 60, VAIT_L1);
			final RecordAttribute ra12c = new RecordAttribute(VAIT_R12, VAIT_R12_C, 60, VAIT_L1);
			final RecordAttribute ra0h2 = new RecordAttribute(VAIT_R0, VAIT_R0_H, 120, VAIT_L2);
			final RecordAttribute ra0c2 = new RecordAttribute(VAIT_R0, VAIT_R0_C, 120, VAIT_L2);
			final RecordAttribute ra1h2 = new RecordAttribute(VAIT_R1, VAIT_R1_H, 120, VAIT_L2);
			final RecordAttribute ra1c2 = new RecordAttribute(VAIT_R1, VAIT_R1_C, 120, VAIT_L2);
			final RecordAttribute ra2h2 = new RecordAttribute(VAIT_R2, VAIT_R2_H, 120, VAIT_L2);
			final RecordAttribute ra2c2 = new RecordAttribute(VAIT_R2, VAIT_R2_C, 120, VAIT_L2);
			final RecordAttribute ra12h2 = new RecordAttribute(VAIT_R12, VAIT_R12_H, 120, VAIT_L2);
			final RecordAttribute ra12c2 = new RecordAttribute(VAIT_R12, VAIT_R12_C, 120, VAIT_L2);

			final IndexAttribute ia1 = new IndexAttribute(VAIT_R0, VAIT_R0_H, 10, VAIT_L2);
			final IndexAttribute ia2 = new IndexAttribute(VAIT_R1, VAIT_R1_H, 10, VAIT_L2);
			final IndexAttribute ia3 = new IndexAttribute(VAIT_R2, VAIT_R2_H, 10, VAIT_L2);
			final IndexAttribute ia4 = new IndexAttribute(VAIT_R12, VAIT_R12_H, 10, VAIT_L2);

			DBTransaction ddlTransactionCreate = dbc.startTransaction(true);

			DmlAssertCommand ac1 = a1.dbAssertCommand(RELVARNAMES.ATTRIBUTE);
			DmlAssertCommand ac2 = a2.dbAssertCommand(RELVARNAMES.ATTRIBUTE);

			DmlCommand.execDmlCommand(ddlTransactionCreate, new DmlMultipleAssignmentCommand(ac1, ac2));

			DmlAssertCommand rc1 = r0.dbAssertCommand(RELVARNAMES.RELVAR);
			DmlAssertCommand rc2 = r1.dbAssertCommand(RELVARNAMES.RELVAR);
			DmlAssertCommand rc3 = r2.dbAssertCommand(RELVARNAMES.RELVAR);
			DmlAssertCommand rc4 = r12.dbAssertCommand(RELVARNAMES.RELVAR);

			DmlAssertCommand kc1 = k0.dbAssertCommand(RELVARNAMES.KEY);
			DmlAssertCommand kc2 = k1.dbAssertCommand(RELVARNAMES.KEY);
			DmlAssertCommand kc3 = k2.dbAssertCommand(RELVARNAMES.KEY);
			DmlAssertCommand kc4 = k12.dbAssertCommand(RELVARNAMES.KEY);

			DmlAssertCommand rac11 = ra0_l1.dbAssertCommand(RELVARNAMES.RELVARATTRIBUTE);
			DmlAssertCommand rac12 = ra0_l2.dbAssertCommand(RELVARNAMES.RELVARATTRIBUTE);
			DmlAssertCommand rac21 = ra1_l1.dbAssertCommand(RELVARNAMES.RELVARATTRIBUTE);
			DmlAssertCommand rac22 = ra1_l2.dbAssertCommand(RELVARNAMES.RELVARATTRIBUTE);
			DmlAssertCommand rac31 = ra2_l1.dbAssertCommand(RELVARNAMES.RELVARATTRIBUTE);
			DmlAssertCommand rac32 = ra2_l2.dbAssertCommand(RELVARNAMES.RELVARATTRIBUTE);
			DmlAssertCommand rac41 = ra12_l1.dbAssertCommand(RELVARNAMES.RELVARATTRIBUTE);
			DmlAssertCommand rac42 = ra12_l2.dbAssertCommand(RELVARNAMES.RELVARATTRIBUTE);

			DmlAssertCommand kac1 = ka0.dbAssertCommand(RELVARNAMES.KEYATTRIBUTE);
			DmlAssertCommand kac2 = ka1.dbAssertCommand(RELVARNAMES.KEYATTRIBUTE);
			DmlAssertCommand kac3 = ka2.dbAssertCommand(RELVARNAMES.KEYATTRIBUTE);
			DmlAssertCommand kac4 = ka12.dbAssertCommand(RELVARNAMES.KEYATTRIBUTE);

			DmlCommand.execDmlCommand(ddlTransactionCreate, new DmlMultipleAssignmentCommand(rc1, rc2, rc3, rc4, kc1, kc2, kc3, kc4, rac11, rac12, rac21, rac22, rac31, rac32, rac41, rac42, kac1, kac2, kac3, kac4));

			DmlAssertCommand fc = f.dbAssertCommand(RELVARNAMES.DBMSFILE);
			DmlAssertCommand hc0s = hrs0.dbAssertCommand(RELVARNAMES.STORAGESPACE);
			DmlAssertCommand hc0h = hrs0.dbAssertCommand(RELVARNAMES.HASHEDRECORDSPACE);
			DmlAssertCommand hc1s = hrs1.dbAssertCommand(RELVARNAMES.STORAGESPACE);
			DmlAssertCommand hc1h = hrs1.dbAssertCommand(RELVARNAMES.HASHEDRECORDSPACE);
			DmlAssertCommand hc2s = hrs2.dbAssertCommand(RELVARNAMES.STORAGESPACE);
			DmlAssertCommand hc2h = hrs2.dbAssertCommand(RELVARNAMES.HASHEDRECORDSPACE);
			DmlAssertCommand hc12s = hrs12.dbAssertCommand(RELVARNAMES.STORAGESPACE);
			DmlAssertCommand hc12h = hrs12.dbAssertCommand(RELVARNAMES.HASHEDRECORDSPACE);
			DmlAssertCommand cc0s = crs0.dbAssertCommand(RELVARNAMES.STORAGESPACE);
			DmlAssertCommand cc0c = crs0.dbAssertCommand(RELVARNAMES.CLUSTEREDRECORDSPACE);
			DmlAssertCommand cc1s = crs1.dbAssertCommand(RELVARNAMES.STORAGESPACE);
			DmlAssertCommand cc1c = crs1.dbAssertCommand(RELVARNAMES.CLUSTEREDRECORDSPACE);
			DmlAssertCommand cc2s = crs2.dbAssertCommand(RELVARNAMES.STORAGESPACE);
			DmlAssertCommand cc2c = crs2.dbAssertCommand(RELVARNAMES.CLUSTEREDRECORDSPACE);
			DmlAssertCommand cc12s = crs12.dbAssertCommand(RELVARNAMES.STORAGESPACE);
			DmlAssertCommand cc12c = crs12.dbAssertCommand(RELVARNAMES.CLUSTEREDRECORDSPACE);

			DmlAssertCommand ccl1 = crsla1.dbAssertCommand(RELVARNAMES.CLUSTEREDRECORDSPACELOCATTRS);
			DmlAssertCommand ccl2 = crsla2.dbAssertCommand(RELVARNAMES.CLUSTEREDRECORDSPACELOCATTRS);
			DmlAssertCommand ccl3 = crsla3.dbAssertCommand(RELVARNAMES.CLUSTEREDRECORDSPACELOCATTRS);
			DmlAssertCommand ccl4 = crsla4.dbAssertCommand(RELVARNAMES.CLUSTEREDRECORDSPACELOCATTRS);

			DmlCommand.execDmlCommand(ddlTransactionCreate, new DmlMultipleAssignmentCommand(fc, hc0s, hc0h, hc1s, hc1h, hc2s, hc2h, hc12s, hc12h, cc0s, cc0c, cc1s, cc1c, cc2s, cc2c, cc12s, cc12c, ccl1, ccl2, ccl3, ccl4));

			DmlAssertCommand crt0h = rt0h.dbAssertCommand(RELVARNAMES.RECORDTYPE);
			DmlAssertCommand crt0c = rt0c.dbAssertCommand(RELVARNAMES.RECORDTYPE);
			DmlAssertCommand crt1h = rt1h.dbAssertCommand(RELVARNAMES.RECORDTYPE);
			DmlAssertCommand crt1c = rt1c.dbAssertCommand(RELVARNAMES.RECORDTYPE);
			DmlAssertCommand crt2h = rt2h.dbAssertCommand(RELVARNAMES.RECORDTYPE);
			DmlAssertCommand crt2c = rt2c.dbAssertCommand(RELVARNAMES.RECORDTYPE);
			DmlAssertCommand crt12h = rt12h.dbAssertCommand(RELVARNAMES.RECORDTYPE);
			DmlAssertCommand crt12c = rt12c.dbAssertCommand(RELVARNAMES.RECORDTYPE);

			DmlAssertCommand cra0h = ra0h.dbAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlAssertCommand cra0c = ra0c.dbAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlAssertCommand cra1h = ra1h.dbAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlAssertCommand cra1c = ra1c.dbAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlAssertCommand cra2h = ra2h.dbAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlAssertCommand cra2c = ra2c.dbAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlAssertCommand cra12h = ra12h.dbAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlAssertCommand cra12c = ra12c.dbAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlAssertCommand cra0h2 = ra0h2.dbAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlAssertCommand cra0c2 = ra0c2.dbAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlAssertCommand cra1h2 = ra1h2.dbAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlAssertCommand cra1c2 = ra1c2.dbAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlAssertCommand cra2h2 = ra2h2.dbAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlAssertCommand cra2c2 = ra2c2.dbAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlAssertCommand cra12h2 = ra12h2.dbAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlAssertCommand cra12c2 = ra12c2.dbAssertCommand(RELVARNAMES.RECORDATTRIBUTE);

			DmlAssertCommand cia1 = ia1.dbAssertCommand(RELVARNAMES.INDEXATTRIBUTE);
			DmlAssertCommand cia2 = ia2.dbAssertCommand(RELVARNAMES.INDEXATTRIBUTE);
			DmlAssertCommand cia3 = ia3.dbAssertCommand(RELVARNAMES.INDEXATTRIBUTE);
			DmlAssertCommand cia4 = ia4.dbAssertCommand(RELVARNAMES.INDEXATTRIBUTE);

			DmlCommand.execDmlCommand(ddlTransactionCreate, new DmlMultipleAssignmentCommand(crt0h, crt0c, crt1h, crt1c, crt2h, crt2c, crt12h, crt12c, cra0h, cra0c, cra1h, cra1c, cra2h, cra2c, cra12h, cra12c, cra0h2, cra0c2, cra1h2, cra1c2, cra2h2, cra2c2, cra12h2, cra12c2, cia1, cia2, cia3, cia4));
			ddlTransactionCreate.end(true);

			// Do the actual testing

			DBTransaction dmlTransaction = dbc.startTransaction(false);

			long tm = System.currentTimeMillis();
			dmlTransaction.execServerCommand(new ServerMultipleAssignmentCommand(new ServerUnassertCommand(VAIT_R0, VAIT_R0), new ServerUnassertCommand(VAIT_R2, VAIT_R2), new ServerUnassertCommand(VAIT_R1, VAIT_R1), new ServerUnassertCommand(VAIT_R12, VAIT_R12)));
			System.out.println("Delete in " + (System.currentTimeMillis() - tm) + " mils."); //$NON-NLS-1$ //$NON-NLS-2$
			tm = System.currentTimeMillis();
			int c = 0, g = totalBatchSize / commitSize;
			StringBuffer insert = new StringBuffer(16384).append(VAIT_R0 + "("); //$NON-NLS-1$
			while (c++ < totalBatchSize) {
				long long1 = (long) (Long.MAX_VALUE * Math.random()), long2 = (long) (Long.MAX_VALUE * Math.random());
				insert.append("TUPLE(" + VAIT_L1 + "(" + long1 + ")" + VAIT_L2 + "(" + long2 + "))"); //$NON-NLS-1$//$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
				if (c % commitSize == 0) {
					insert.append(")"); //$NON-NLS-1$
					ServerAssignmentCommand c1 = new ServerAssertCommand(VAIT_R0, insert.toString());
					ServerAssignmentCommand c2 = new ServerAssertCommand(VAIT_R1, insert.toString());
					ServerAssignmentCommand c3 = new ServerAssertCommand(VAIT_R2, insert.toString());
					ServerAssignmentCommand c4 = new ServerAssertCommand(VAIT_R12, insert.toString());
					tm = System.currentTimeMillis();
					dmlTransaction.execServerCommand(new ServerMultipleAssignmentCommand(c1, c2, c3, c4));
					System.out.println(EngineVolumeSpeedTestProperties.COMMITSIZEDEFAULT + "x4 (" + (g--) + ")added in " + (System.currentTimeMillis() - tm) + " mils."); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					tm = System.currentTimeMillis();
					dmlTransaction.commit();
					System.out.println("commit in " + (System.currentTimeMillis() - tm) + " mils."); //$NON-NLS-1$ //$NON-NLS-2$
					insert = new StringBuffer(16384).append(VAIT_R0 + "("); //$NON-NLS-1$
				}
			}

			tm = System.currentTimeMillis();
			dmlTransaction.execServerCommand(new ServerMultipleAssignmentCommand(new ServerUnassertCommand(VAIT_R0, VAIT_R0), new ServerUnassertCommand(VAIT_R2, VAIT_R2), new ServerUnassertCommand(VAIT_R1, VAIT_R1), new ServerUnassertCommand(VAIT_R12, VAIT_R12)));
			System.out.println("Cleanup in " + (System.currentTimeMillis() - tm) + " mils."); //$NON-NLS-1$ //$NON-NLS-2$
			tm = System.currentTimeMillis();
			dmlTransaction.end(true);
			System.out.println("commit in " + (System.currentTimeMillis() - tm) + " mils."); //$NON-NLS-1$ //$NON-NLS-2$

			// Start cleaning up the test case from the catalog

			DmlUnassertCommand drt0h = rt0h.dbUnAssertCommand(RELVARNAMES.RECORDTYPE);
			DmlUnassertCommand drt0c = rt0c.dbUnAssertCommand(RELVARNAMES.RECORDTYPE);
			DmlUnassertCommand drt1h = rt1h.dbUnAssertCommand(RELVARNAMES.RECORDTYPE);
			DmlUnassertCommand drt1c = rt1c.dbUnAssertCommand(RELVARNAMES.RECORDTYPE);
			DmlUnassertCommand drt2h = rt2h.dbUnAssertCommand(RELVARNAMES.RECORDTYPE);
			DmlUnassertCommand drt2c = rt2c.dbUnAssertCommand(RELVARNAMES.RECORDTYPE);
			DmlUnassertCommand drt12h = rt12h.dbUnAssertCommand(RELVARNAMES.RECORDTYPE);
			DmlUnassertCommand drt12c = rt12c.dbUnAssertCommand(RELVARNAMES.RECORDTYPE);

			DmlUnassertCommand dra0h = ra0h.dbUnAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlUnassertCommand dra0c = ra0c.dbUnAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlUnassertCommand dra1h = ra1h.dbUnAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlUnassertCommand dra1c = ra1c.dbUnAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlUnassertCommand dra2h = ra2h.dbUnAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlUnassertCommand dra2c = ra2c.dbUnAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlUnassertCommand dra12h = ra12h.dbUnAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlUnassertCommand dra12c = ra12c.dbUnAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlUnassertCommand dra0h2 = ra0h2.dbUnAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlUnassertCommand dra0c2 = ra0c2.dbUnAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlUnassertCommand dra1h2 = ra1h2.dbUnAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlUnassertCommand dra1c2 = ra1c2.dbUnAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlUnassertCommand dra2h2 = ra2h2.dbUnAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlUnassertCommand dra2c2 = ra2c2.dbUnAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlUnassertCommand dra12h2 = ra12h2.dbUnAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlUnassertCommand dra12c2 = ra12c2.dbUnAssertCommand(RELVARNAMES.RECORDATTRIBUTE);

			DmlUnassertCommand dia1 = ia1.dbUnAssertCommand(RELVARNAMES.INDEXATTRIBUTE);
			DmlUnassertCommand dia2 = ia2.dbUnAssertCommand(RELVARNAMES.INDEXATTRIBUTE);
			DmlUnassertCommand dia3 = ia3.dbUnAssertCommand(RELVARNAMES.INDEXATTRIBUTE);
			DmlUnassertCommand dia4 = ia4.dbUnAssertCommand(RELVARNAMES.INDEXATTRIBUTE);

			DBTransaction ddlTransactionDestroy = dbc.startTransaction(true);
			DmlCommand.execDmlCommand(ddlTransactionDestroy, new DmlMultipleAssignmentCommand(drt0h, drt0c, drt1h, drt1c, drt2h, drt2c, drt12h, drt12c, dra0h, dra0c, dra1h, dra1c, dra2h, dra2c, dra12h, dra12c, dra0h2, dra0c2, dra1h2, dra1c2, dra2h2, dra2c2, dra12h2, dra12c2, dia1, dia2, dia3, dia4));

			DmlUnassertCommand dcl1 = crsla1.dbUnAssertCommand(RELVARNAMES.CLUSTEREDRECORDSPACELOCATTRS);
			DmlUnassertCommand dcl2 = crsla2.dbUnAssertCommand(RELVARNAMES.CLUSTEREDRECORDSPACELOCATTRS);
			DmlUnassertCommand dcl3 = crsla3.dbUnAssertCommand(RELVARNAMES.CLUSTEREDRECORDSPACELOCATTRS);
			DmlUnassertCommand dcl4 = crsla4.dbUnAssertCommand(RELVARNAMES.CLUSTEREDRECORDSPACELOCATTRS);

			DmlUnassertCommand fd = f.dbUnAssertCommand(RELVARNAMES.DBMSFILE);
			DmlUnassertCommand hd0s = hrs0.dbUnAssertCommand(RELVARNAMES.STORAGESPACE);
			DmlUnassertCommand hd0h = hrs0.dbUnAssertCommand(RELVARNAMES.HASHEDRECORDSPACE);
			DmlUnassertCommand hd1s = hrs1.dbUnAssertCommand(RELVARNAMES.STORAGESPACE);
			DmlUnassertCommand hd1h = hrs1.dbUnAssertCommand(RELVARNAMES.HASHEDRECORDSPACE);
			DmlUnassertCommand hd2s = hrs2.dbUnAssertCommand(RELVARNAMES.STORAGESPACE);
			DmlUnassertCommand hd2h = hrs2.dbUnAssertCommand(RELVARNAMES.HASHEDRECORDSPACE);
			DmlUnassertCommand hd12d = hrs12.dbUnAssertCommand(RELVARNAMES.STORAGESPACE);
			DmlUnassertCommand hd12h = hrs12.dbUnAssertCommand(RELVARNAMES.HASHEDRECORDSPACE);
			DmlUnassertCommand dc0s = crs0.dbUnAssertCommand(RELVARNAMES.STORAGESPACE);
			DmlUnassertCommand dc0c = crs0.dbUnAssertCommand(RELVARNAMES.CLUSTEREDRECORDSPACE);
			DmlUnassertCommand dc1s = crs1.dbUnAssertCommand(RELVARNAMES.STORAGESPACE);
			DmlUnassertCommand dc1c = crs1.dbUnAssertCommand(RELVARNAMES.CLUSTEREDRECORDSPACE);
			DmlUnassertCommand dc2s = crs2.dbUnAssertCommand(RELVARNAMES.STORAGESPACE);
			DmlUnassertCommand dc2c = crs2.dbUnAssertCommand(RELVARNAMES.CLUSTEREDRECORDSPACE);
			DmlUnassertCommand dc12s = crs12.dbUnAssertCommand(RELVARNAMES.STORAGESPACE);
			DmlUnassertCommand dc12c = crs12.dbUnAssertCommand(RELVARNAMES.CLUSTEREDRECORDSPACE);

			DmlCommand.execDmlCommand(ddlTransactionDestroy, new DmlMultipleAssignmentCommand(fd, hd0s, hd0h, hd1s, hd1h, hd2s, hd2h, hd12d, hd12h, dc0s, dc0c, dc1s, dc1c, dc2s, dc2c, dc12s, dc12c, dcl1, dcl2, dcl3, dcl4));

			DmlUnassertCommand kad21 = ka0.dbUnAssertCommand(RELVARNAMES.KEYATTRIBUTE);
			DmlUnassertCommand kad22 = ka1.dbUnAssertCommand(RELVARNAMES.KEYATTRIBUTE);
			DmlUnassertCommand kad23 = ka2.dbUnAssertCommand(RELVARNAMES.KEYATTRIBUTE);
			DmlUnassertCommand kad24 = ka12.dbUnAssertCommand(RELVARNAMES.KEYATTRIBUTE);

			DmlUnassertCommand rad1 = ra0_l1.dbUnAssertCommand(RELVARNAMES.RELVARATTRIBUTE);
			DmlUnassertCommand rad2 = ra0_l2.dbUnAssertCommand(RELVARNAMES.RELVARATTRIBUTE);
			DmlUnassertCommand rad3 = ra1_l1.dbUnAssertCommand(RELVARNAMES.RELVARATTRIBUTE);
			DmlUnassertCommand rad4 = ra1_l2.dbUnAssertCommand(RELVARNAMES.RELVARATTRIBUTE);
			DmlUnassertCommand rad5 = ra2_l1.dbUnAssertCommand(RELVARNAMES.RELVARATTRIBUTE);
			DmlUnassertCommand rad6 = ra2_l2.dbUnAssertCommand(RELVARNAMES.RELVARATTRIBUTE);
			DmlUnassertCommand rad7 = ra12_l1.dbUnAssertCommand(RELVARNAMES.RELVARATTRIBUTE);
			DmlUnassertCommand rad8 = ra12_l2.dbUnAssertCommand(RELVARNAMES.RELVARATTRIBUTE);

			DmlUnassertCommand kd1 = k0.dbUnAssertCommand(RELVARNAMES.KEY);
			DmlUnassertCommand kd2 = k1.dbUnAssertCommand(RELVARNAMES.KEY);
			DmlUnassertCommand kd3 = k2.dbUnAssertCommand(RELVARNAMES.KEY);
			DmlUnassertCommand kd4 = k12.dbUnAssertCommand(RELVARNAMES.KEY);

			DmlUnassertCommand rd1 = r0.dbUnAssertCommand(RELVARNAMES.RELVAR);
			DmlUnassertCommand rd2 = r1.dbUnAssertCommand(RELVARNAMES.RELVAR);
			DmlUnassertCommand rd3 = r2.dbUnAssertCommand(RELVARNAMES.RELVAR);
			DmlUnassertCommand rd4 = r12.dbUnAssertCommand(RELVARNAMES.RELVAR);

			DmlCommand.execDmlCommand(ddlTransactionDestroy, new DmlMultipleAssignmentCommand(kad21, kad22, kad23, kad24, rad1, rad2, rad3, rad4, rad5, rad6, rad7, rad8, kd1, kd2, kd3, kd4, rd1, rd2, rd3, rd4));

			DmlUnassertCommand ad1 = a1.dbUnAssertCommand(RELVARNAMES.ATTRIBUTE);
			DmlUnassertCommand ad2 = a2.dbUnAssertCommand(RELVARNAMES.ATTRIBUTE);

			DmlCommand.execDmlCommand(ddlTransactionDestroy, new DmlMultipleAssignmentCommand(ad1, ad2));
			ddlTransactionDestroy.end(true);

		} catch (ConnectionClosedException e) {
			fail(e.getClass().getName() + ' ' + e.getMessage());
		} catch (ErrorMessageException e) {
			fail(e.getClass().getName() + ' ' + e.getMessage());
		} finally {
			dbc.close();
		}
	}
}
